import Cryptodome.PublicKey.RSA
from Cryptodome.PublicKey import RSA
from Cryptodome.Signature import pkcs1_15
from Cryptodome.Hash import SHA256

class rsa:


    def __init__(self, private_key = None):
        if private_key == None:
            self.private_key, self.public_key  = self.generate_key_pair()
        else:
            self.private_key, self.public_key = self.generate_from_existing_key(private_key)

    def generate_from_existing_key(self, private_key):
        private_key = RSA.import_key(private_key)
        public_key = private_key.public_key()
        return private_key, public_key

    def test_if_key_is_right(self):
        if type(self.public_key) != RSA.RsaKey:
            self.public_key = Cryptodome.PublicKey.RSA.importKey(self.public_key)
        if type(self.private_key) != RSA.RsaKey:
            self.private_key = Cryptodome.PublicKey.RSA.import_key(self.private_key)
        print(type(self.private_key))

    def return_keys(self):
        return self.private_key, self.public_key

    def return_string_keys(self):
        return self.private_key.export_key().decode('utf-8'), self.public_key.export_key().decode('utf-8')

    def generate_key_pair(self):
        key = RSA.generate(2048)
        private_key = key
        public_key = key.publickey()
        return private_key, public_key

    def rsa_sign(self, plaintext):
        self.test_if_key_is_right()
        h = SHA256.new(plaintext.encode('utf-8'))
        signature = pkcs1_15.new(self.private_key).sign(h)
        return signature

    def rsa_verify(self, plaintext, signature):
        self.test_if_key_is_right()
        h = SHA256.new(plaintext.encode('utf-8'))
        try:
            pkcs1_15.new(self.public_key).verify(h, signature)
            return True
        except (ValueError, TypeError):
            return False


from . import terminal_exe


class test_class:

    def __init__(self):
        self.yes = False

    def build(self, ter, args):
        self.yes = True

    def run(self, ter, args):
        if self.yes == True:
            ter.t_print('working')

    def display(self, term: terminal_exe.Terminal_commands, args):
        display = {'list': {'1': '1', '2': '2', '3': '3'}, 'nolist': 'nolist'}
        ter_display = terminal_exe.Termianl_Information(display, term.screen)
        ter_display.run()


wow = 'wow'

count = 0

def info():
    return {'a': {'a': None, 'b': None, 'c': None}, 'b': None, 'c': None, f'{count}': None}


def get_info(ter, args):
    new_info = info()
    ter_display = terminal_exe.Termianl_Information(new_info, ter.screen)
    ter_display.run()




def help(ter, args):
    ter.t_print('help')


e1 = test_class()

def example1():

    terminal1 = terminal_exe.Terminal_commands({}, display_info="Type 'help' to see all commands or type 'exit' to exit")
    terminal1.add_commands(wow=wow, help=help, build=e1.build, run=e1.run, display=e1.display)
    terminal1.add_commands(terminal=get_info)
    controller = terminal_exe.Controller()
    controller.set_start(terminal1)
    controller.run()

example1()


import os
from pathlib import Path
import webbrowser

def get_current_directory():
    return Path.cwd()

def get_current_file_name():
    return os.path.basename(__file__)

def path_join(path1, path2):
    return os.path.join(path1, path2)

def open_browser_with_url(url:str):
    webbrowser.open(url)

def if_directory_not_exists(dir:str):
    if not os.path.exists(dir):
        os.makedirs(dir)

def string_in_list(string:str):
    List = string.split(', ')
    List[0] = List[0][List[0].find('[')+1:]
    List[-1] = List[-1][:List[-1].find(']')]
    return List


class comunication:
    Information = {

    }

    def write(self, **kwargs):
        for x in kwargs:
            self.Information[x] = kwargs[x]

    def get(self, key):
        return self.Information[key]




import datetime
from dateutil.relativedelta import relativedelta


def format_now(date):
    datetime_object = datetime.datetime.strptime(date, '%y-%m-%d %H:%M:%S')
    return datetime_object
def format_all(date):
    datetime_object = datetime.datetime.strptime(date, '%d-%m-%y %H:%M:%S')
    return datetime_object

def format(date):
    try:
        datetime_object = datetime.datetime.strptime(date, '%d-%m-%y')
        return datetime_object
    except ValueError:
        try:
            datetime_object = datetime.datetime.strptime(date, '%d-%m-%y %H:%M:%S')
            return datetime_object
        except ValueError:
            try:
                datetime_object = datetime.datetime.strptime(date, '%y-%m-%d %H:%M:%S')
                return datetime_object
            except ValueError:
                try:
                    datetime_object = datetime.datetime.strptime(date, '%Y-%m-%d %H:%M:%S.%f')
                    return datetime_object
                except ValueError:
                    try:
                        datetime_object = datetime.datetime.strptime(date, '%Y-%m-%d %H:%M:%S')
                        return datetime_object
                    except Exception:
                        print(date)

def current():
    now = datetime.datetime.now()
    return now

def add(date, months:int=0, days:int=0, years:int=0, minutes:int=0, seconds:int=0, hours:int=0):
    if months != 0:
        return date + relativedelta(months=months)
    if days != 0:
        return date + relativedelta(days=days)
    if years != 0:
        return date + relativedelta(years=years)
    if minutes != 0:
        return date + relativedelta(minutes=minutes)
    if seconds != 0:
        return date + relativedelta(seconds=seconds)
    if hours != 0:
        return date + relativedelta(hours=hours)

def covert_relativtimedelta(date):
    relative_date = relativedelta(years=date.year, months=date.month, days=date.day, hours=date.hour,
                                  minutes=date.minute, seconds=date.second, microseconds=date.microsecond)
    return relative_date

def datetime_to_String(date):
    part1 = list()
    part2 = list()
    if date.microsecond != 0:
        part1.append(str(date.microsecond))
    if date.second != 0:
        part1.append(str(date.second))
    if date.minute != 0:
        part1.append(str(date.minute))
    if date.hour != 0:
        part1.append(str(date.hour))
    part1 = part1[::-1]
    part1 = ':'.join(part1)
    if date.day != 0:
        part2.append(str(date.day))
    if date.month != 0:
        part2.append(str(date.month))
    if date.year != 0:
        part2.append(str(date.year))
    part2 = part2[::-1]
    part2 = '-'.join(part2)
    if part1 == '':
        return part2
    else:
        return ' '.join([part2, part1])



from .. import basic_features
from . import html
import os

class graph():

    def __init__(self, labels:list, data:dict, name:str = 'myChart', type ='line'):
        basic_features.if_directory_not_exists('html')
        self.labels = labels
        self.data = data
        self.name = name
        self.type = type

    def basic_view(self, width_height = (100, 100), title ='Chart.js Line Chart'):
        script1, html_script, script2 = html.charts.full_build(self.labels, self.data, self.name, width_height, title, self.type)
        basic_html = f'''<!DOCTYPE html><html lang="en"><head><meta charset="UTF-8"><title>Title</title>{script1}</head><body>{html_script}{script2}</body></html>'''
        path = basic_features.get_current_directory()
        path = os.path.join(path, 'html', 'view.html')
        with open(path, 'w') as file:
            file.write(basic_html)
        basic_features.open_browser_with_url(path)



import curses
import asyncio
from abc import ABC
import abc
import time

#controller to controll and build the terminal
class Controller():

    def __init__(self):
        #creates the asyncio event loop
        self.build = {}

    def run(self):
        self.build['START'].run()

    def set_start(self, info):
        self.build['START'] = info



class base_terminal:


    def screen_setup(self, screen):
        self.running = False
        if screen == None:
            self.screen = curses.initscr()
        else:
            self.screen = screen

    def display(self):
        raise NotImplementedError("Please Implement the method display")

    def run(self):
        raise NotImplementedError("Please Implement run")

    def start(self):
        if not self.running:
            self.running = True

    def stop(self):
        if self.running:
            self.running = False

    def conv_func(self, func):
        result = str(func)
        if result == None:
            return False
        else:
            return result

    def conv_str(self, STR):
        return STR

#this is a terminal used to display simple information
class Termianl_Information(base_terminal):

    #inits the
    def __init__(self, path, screen = None):
        #inits the screen, can be given by the parent class and reuses the screen
        self.screen_setup(screen)
        #setting for the terminal
        curses.noecho()
        curses.cbreak()
        #this is the whole path for the display of information
        self.path : dict = path
        self.path['EXIT'] = 'EXIT'
        self.path_conv()
        #setups the current path, this is how the programm knows where in the path we are
        self.current_path : list = [str(x) for x in self.path.keys()]
        self.current_path : list = [self.current_path[0]]
        #this is just the path info for the current path(= where we currently are) is saved
        self.current_path_info : dict = self.get_current_path_info()
        #this is the current
        self.current : str = str(self.current_path[0])
        self.current_index : int = 0
        self.current_max : int = len(self.get_current_path_info().keys()) - 1

    def setup_current(self):
        self.current = str([x for x in self.current_path_info][self.current_index])

    def get_current_path_info(self, search_path = None):
        path = self.path
        if search_path == None:
            current = [x for x in self.current_path]
        else:
            current = [x for x in search_path]
        del current[0]
        if not current == []:
            for x in current:
                path = path[x]
                path['BACK'] = 'BACK'
        else:
            path = self.path
        return path

    def move(self, move_UP:bool):
        if move_UP and self.current_index != 0:
            self.current_index -= 1
        elif not move_UP and self.current_index != self.current_max:
            self.current_index += 1
        self.setup_current()
        self.display()

    #goes to the next in the path
    def next(self):
        self.current_path.append(self.current)
        self.current_index = 0
        self.current = str(self.current_path[0])
        self.current_path_info = self.get_current_path_info()
        self.current_max = len(self.current_path_info.keys()) - 1

    #goes back in the current path
    def back(self):
        del self.current_path[len(self.current_path)-1]
        self.current_path_info = self.get_current_path_info()
        self.current = self.current_path[len(self.current_path)-1]
        self.current_index = [x for x in self.current_path_info].index(self.current)
        self.current_max = len(self.current_path_info.keys()) - 1

    def use(self):
        value = self.current_path_info[self.current]
        if value == 'BACK':
            self.back()
        elif value == 'EXIT':
            return 0
        elif isinstance(value, dict):
            self.next()
        self.display()

    def path_conv(self):

        def costum_path_info(path_path):
            path = self.path
            print(path_path)
            for x in path_path:
                path = path[x]
            return path

        path_list = []
        for x in self.path.keys():
            path_list.append([x])
        del path_list[path_list.index(['EXIT'])]



        while True:


            if path_list == []:
                break

            current = path_list[0]

            current_info = costum_path_info(current)

            if isinstance(current_info, dict):
                for x in current_info.keys():
                    current_copy = [b for b in current]
                    current_copy.append(x)
                    path_list.append(current_copy)

            elif isinstance(current_info, list):
                new_info = dict()
                for x in current_info:
                    new_info[str(x)] = None

                self.add_path_info(current, new_info)

            elif callable(current_info):
                result = current_info()
                self.add_path_info(current, result)
                path_list.append(current)


            del path_list[0]

    def add_path_info(self, path, info):
        new_path = self.path
        last_char = path[len(path) - 1]
        del path[len(path) - 1]
        for x in path:
            new_path = new_path[x]
        new_path[last_char] = info


    def display(self):
        self.screen.clear()
        info = self.current_path_info
        self.screen.addstr(0, 0, 'TAB to exit')
        for x in range(len(info)):
            if x == self.current_index:
                space = ">> "
            else:
                space = "   "
            self.screen.addstr(x+2, 0, f'{space}{[b for b in info.keys()][x]}')

        self.screen.refresh()

    def run(self):
        self.start()
        while True:
            self.display()
            c = self.screen.getch()
            if c == 9:
                break
            if c == 10:
                output = self.use()
                if output == 0:
                    break
            if c == ord('w') or c == ord('W'):
                self.move(True)
            if c == ord('s') or c == ord('S'):
                self.move(False)
        self.stop()


def terminal_command(self, *args):
    def do_func(func):
        self.commands[f"{func.__name__}"] = func
        return func

    return do_func

class Terminal_commands(base_terminal):

    def __init__(self, commands:dict ,  screen = None, display_info:str = ''):
        self.screen_setup(screen)
        self.history = []
        self.commands : dict = {}
        self.implement_commands(commands)
        self.setup_commands()
        self.screen = curses.initscr()
        curses.echo()
        curses.cbreak()
        self.screen.keypad(True)
        self.tracknum : int = 0
        self.display_info = display_info

    def add_commands(self, **kwargs):
        for x in kwargs:
            self.commands[f"i.{x}"] = kwargs[x]

    def __call__(self, *args, **kwargs):
        print(self)

    def implement_commands(self, commands : dict):
        for x in commands:
            self.commands[f"i.{x}"] = commands[x]

    def display(self, ignore_input:bool=False):
        self.screen.clear()
        tracknum : int = 0
        for x in range(len(self.history)):
            self.screen.addstr(x, 0, self.history[x])
            tracknum = x
        self.screen.addstr(tracknum, 0, "")
        if ignore_input == False:
            self.screen.addstr(tracknum + 1, 0, ">> ")
        else:
            self.screen.addstr(tracknum + 1, 0, "")
        self.tracknum = tracknum + 1
        self.screen.refresh()

    def run(self):
        if self.history == []:
            self.history.append(self.display_info)
        # Initialisieren von curses
        stdscr = self.screen
        self.display()
        # Hauptterminal-Schleife
        while True:
            self.display()
            user_input = stdscr.getstr(self.tracknum, 3, 60)
            self.screen.refresh()
            user_input = user_input.decode("utf-8").split()
            if user_input:
                command = user_input[0]
                self.t_print(f'>  {command}')
                if command in self.commands:
                    output = self.commands[command]
                    if isinstance(output, str):
                        self.t_print(output)
                    else:
                        output = output(self, user_input[1:])
                        curses.echo()
                        if output == 0:
                            break
                else:
                    self.t_print(f'Unknown Command "{command}"')

            # Warten auf Benutzereingabe

    def yes_no_question(self, input:str=''):
        self.t_print(input)
        self.t_print('y[es] or n[o]')
        self.display(True)
        while True:
            key = self.screen.getch()
            if key == ord('y') or key == ord('Y'):
                return True
            if key == ord('n') or key == ord('N'):
                return False

    def inputs(self, eingabe:dict):
        self.screen.clear()
        tracknum = 0
        for x in eingabe:
            if eingabe[x] == 'HIDDEN':
                curses.noecho()
            self.screen.addstr(tracknum, 0, f'{x}:')
            user_input = self.screen.getstr(tracknum, 1 + len(x), 60)
            self.screen.refresh()
            user_input = user_input.decode("utf-8").split()
            curses.echo()
            eingabe[x] = user_input[0]
            tracknum += 1
        return eingabe

    def t_print(self, info:str):
        self.history.append(info)
        self.display()

    def setup_commands(self):

        @terminal_command(self)
        def clear(self:Terminal_commands, args):
            self.history = []
            self.display()

        @terminal_command(self)
        def exit(self:Terminal_commands, args):
            return 0

        @terminal_command(self)
        def help(self:Terminal_commands, args):
            self.t_print('')
            self.t_print('Alle möglichen Befehle:')
            for x in self.commands:
                self.t_print(f'     {x}')
            self.t_print('')

class No_func(Exception):

    pass

class use_func():

    def __init__(self):
        self.terminal = None
        self.args :list = []


    def run(self, terminal, args):
        self.terminal = terminal
        self.args = args
        self.func()

    def func(self):
        raise NotImplemented

class Builder:

    def __init__(self, path:dict={}):
        self.build = path

    def Return(self):
        return self.build







import time
import mysql.connector
import datetime
from . import dates
import multiprocessing
import sqlite3


modes = ['mysql', 'sqlite']

class SQL_Class:

    def __init__(self, mode='mysql'):
        self.mode = mode
        self.db = None
        self.cursor = None
        self.tabels = []
        self.database = ''
        self.host = 'localhost'
        self.user = ''
        self.password = ''

    def __del__(self):
        self.cursor.close()
        self.db.close()

    def login(self, user:str='', password:str='', database:str='', tables:list='', host_name:str = 'localhost'):
        self.host = host_name
        self.password = password
        self.user = user
        self.database = database
        self.tabels = tables
        if self.mode == modes[0]:
            self.connect_mysql()
        if self.mode == modes[1]:
            self.connect_sqlite()

    def connect_mysql(self):
        self.db = mysql.connector.connect(
            host=self.host,
            user=self.user,
            password=self.password,
            database=self.database,
        )
        self.cursor = self.db.cursor()

    def connect_sqlite(self):
        self.db = sqlite3.connect(self.database)
        self.cursor = self.db.cursor()

    def ifconnected(self):
        if self.cursor == '':
            return False
        else:
            return True

    def reset_auto_increment(self):
        self.Execute_SQL_Command(f'ALTER TABLE `{self.database}`.`{self.tabels[0]}`  AUTO_INCREMENT = 1')
        self.db.commit()

    def clear_table(self):
        self.Execute_SQL_Command(f'DELETE FROM `{self.database}`.`{self.tabels[0]}`')
        self.db.commit()

    def basic_read(self, tabels=None, *args, **kwargs):
        if not self.ifconnected():
            raise NotConnected
        if tabels is None:
            tabels = [x for x in self.tabels]
        for x in range(len(tabels)):
            tabels[x] = f'{self.database}.{tabels[x]}'
        tabels_str = ', '.join(tabels)
        if len(args) == 0:
            select = '*'
        else:
            select = ', '.join(args)
        elements = []
        for x in kwargs:
            elements.append(f"{x} = '{kwargs[x]}'")
        elements = ' and '.join(elements)
        if elements == '':
            Abfrage = f'SELECT {select} FROM {tabels_str}'
        else:
            Abfrage = f'SELECT {select} FROM {tabels_str} WHERE {elements}'
        Return = self.Execute_SQL_Command(Abfrage)
        return Return


    def basic_write(self, tabels=None, **kwargs):
        if not self.ifconnected():
            raise NotConnected
        if tabels is None:
            tabels = [x for x in self.tabels]
        for x in range(len(tabels)):
            tabels[x] = f'{self.database}.{tabels[x]}'
        tabels_str = tabels[0]
        colums = []
        for x in kwargs:
            colums.append(x)
        values = []
        for x in kwargs:
            values.append(kwargs[x])
        for x in range(len(values)):
            values[x] = f"'{values[x]}'"
        colums = ', '.join(colums)
        values = ', '.join(values)
        Abfrage = f'INSERT INTO {tabels_str} ({colums}) VALUES ({values})'
        self.Execute_SQL_Command(Abfrage)
        self.db.commit()

    def basic_delete(self, tabels=None, **kwargs):
        if tabels is None:
            tabels = [x for x in self.tabels]
        for x in range(len(tabels)):
            tabels[x] = f'{self.database}.{tabels[x]}'
        print(tabels)
        tabels_str = tabels[0]
        elements = list()
        for x in kwargs:
            elements.append(f"{x} = '{kwargs[x]}'")
        elements = ' and '.join(elements)
        Abfrage = f'DELETE FROM {tabels_str} WHERE {elements}'
        self.Execute_SQL_Command(Abfrage)
        self.db.commit()

    def Execute_SQL_Command(self, command:str):
        print(command)
        self.cursor.execute(command)
        return self.cursor.fetchall()


class NotConnected(Exception):

    pass
class auto_delete():

    def __init__(self, tabel:str, colum:str, Buffertime:list, SQL:SQL_Class = None): # Buffertime: [name:(days, months, years, minutes, seconds, hours), time:int]
        self.SQL = SQL
        self.tabel = tabel
        self.colum = colum
        self.Buffertime = Buffertime
        self.Process = None
        self.setup()

    def setup(self):
        self.SQL.tabels = [self.tabel]
        self.Process = multiprocessing.Process(target=self.exe)

    def delete_if_too_old_date(self):
        dates_list = self.SQL.basic_read(None, self.colum)
        dates_list = [dates.format(x[0]) for x in dates_list]
        dates_delete = []
        for x in dates_list:
            if 'months' == self.Buffertime[0]:
                if dates.add(x, months=self.Buffertime[1]) < dates.current():
                    dates_delete.append(x)
            if 'days' == self.Buffertime[0]:
                if dates.add(x, days=self.Buffertime[1]) < dates.current():
                    dates_delete.append(x)
            if 'years' == self.Buffertime[0]:
                if dates.add(x, years=self.Buffertime[1]) < dates.current():
                    dates_delete.append(x)
            if 'minutes' == self.Buffertime[0]:
                if dates.add(x, minutes=self.Buffertime[1]) < dates.current():
                    dates_delete.append(x)
            if 'seconds' == self.Buffertime[0]:
                if dates.add(x, seconds=self.Buffertime[1]) < dates.current():
                    dates_delete.append(x)
            if 'hours' == self.Buffertime[0]:
                if dates.add(x, hours=self.Buffertime[1]) < dates.current():
                    dates_delete.append(x)
        print(dates_delete)
        for x in dates_delete:
            self.SQL.Execute_SQL_Command(f"DELETE FROM `{self.SQL.database}`.`{self.tabel}` WHERE (`{self.colum}` = '{x}')")
        self.SQL.db.commit()

    def exe(self):
        while True:
            try:
                self.delete_if_too_old_date()
            except Exception:
                print(Exception)

    def run(self):
        multiprocessing.freeze_support()
        self.Process.start()

    def stop(self):
        self.Process.terminate()



colors = [
    "#FF6384",  # Rotes Rosa
    "#36A2EB",  # Blaues Türkis
    "#FFCE56",  # Gelb
    "#4BC0C0",  # Türkis
    "#9966FF",  # Lila
    "#FF8A80",  # Hellrot
    "#1E88E5",  # Dunkles Blau
    "#FFD54F",  # Orange
    "#4DB6AC",  # Grünblau
    "#7CB342",  # Grün
    "#F06292",  # Rosa
    "#9575CD",  # Hellviolett
    "#64B5F6",  # Himmelblau
    "#FFB74D",  # Orange
    "#81C784",  # Hellgrün
    "#7986CB",  # Blauviolett
    "#A1887F",  # Braungrau
    "#90A4AE",  # Blaugrau
]

def full_build(labels:list, data:dict, name:str = 'myChart', width_height = (100, 100), title ='Chart.js Line Chart', type ='line'):
    return '<script src="https://cdn.jsdelivr.net/npm/chart.js"></script>', f'<canvas id="{name}" width="{width_height[0]}" height="{width_height[1]}"></canvas>',f'''<script>const xValues = {labels}; const data = {build_data(data)}; const config = {build_config(title, type)}; {name} = new Chart("{name}", config); </script>'''

def make_colors(data:dict):
    color_num = 0
    for x in data.keys():
        if color_num == len(colors):
            color_num = 0
        data[x].append(colors[color_num])
        color_num += 1

def build_data(data:dict):
    make_colors(data)
    datas = list()
    for x in data.keys():
        js_data = f'''{{label: '{x}', data: {data[x][0:-1]}, borderColor: "{data[x][-1]}", fill: false}}'''
        datas.append(js_data)
    datas = ', '.join(datas)
    full_data = f'''{{labels: xValues, datasets: [{datas}]}}'''
    return full_data

def build_config(title = 'Chart.js Line Chart', type = 'line'):
    config_data = f'''{{type: '{type}', data: data, options: {{ responsive: true, plugins: {{ legend: {{ position: 'top', }}, title: {{ display: true, text: '{title}' }} }} }}, }};'''
    return config_data



from .. import data
from .. import dates

class process_data:

    def __init__(self, config:dict, **kwargs):
        self.data:dict = self.clean_data(kwargs)
        self.num_data:int = 0
        self.config(config)
        self.data, self.labels = self.process()

    def get(self):
        return self.labels, self.data

    def config(self, config:dict):
        keys = config.keys()
        if 'time' in keys:
            self.time = config['time']

    def clean_data(self, data:dict):
        for x in [x for x in data.keys()]:
            data[x] = sorted(data[x], key=lambda x: x[-1])
        return data

    def process_dates(self):
        min_data, max_data = self.seperate_dates()
        dates = [min_data]
        while dates[-1] < max_data:
            dates.append(self.time+dates[-1])
        return dates

    def process(self):
        _data = dict()
        dates = self.process_dates()
        print(dates)
        for x in self.data.keys():
            _data[x] = self._process(self.data[x], dates)
        for x in _data.keys():
            _data[x] = self.do_data(_data[x])
            print(_data[x])
        del dates[-1]
        return _data, self.cov_dates_to_str(dates)


    def do_data(self, data):
        values = list()
        for x in data:
            values.append(self._do_data(x))
        return values

    def _do_data(self, data):
        values = list()
        func = data[0][0][-1]
        for x in data:
            _values = list()
            for xx in x:
                _values.append(xx[0])
            values.append(func(_values))
        return func(values)

    def _process(self, data:list, dates_list):
        _data = [x for x in data]
        finished_data = []
        for num in range(len(dates_list)):
            if num != (len(dates_list) - 1):
                date = dates_list[num + 1]
                _finished_data = []
                for x in _data:
                    if x[-1] < date:
                        _finished_data.append(x[0:-1])
                if len(_finished_data) != 0:
                    for x in range(len(_finished_data)):
                        del _data[0]
                finished_data.append(_finished_data)
        return finished_data

    def seperate_dates(self):
        data = self.data
        values = sorted([item[-1] for sublist in data.values() for item in sublist])
        return values[0], values[-1]

    def cov_dates_to_str(self, date_list:list):
        for x in range(len(date_list)):
            date_list[x] = dates.datetime_to_String(date_list[x])
        return date_list

